/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.commons.util.debug.DebugLevel;
import fr.amap.commons.util.debug.Debugger;
import fr.amap.commons.util.swing.FrameView;
import fr.amap.commons.util.swing.jprogress.JProgressFrame;
import fr.amap.lidar.amapvox.engine3d.entities.util.ViewType;
import java.awt.FlowLayout;
import java.io.File;
import java.io.FileNotFoundException;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.jdesktop.swingx.JXButton;
import fr.amap.lidar.amapvox.visualization.controllers.ToolbarController;
import java.awt.Dimension;
import javax.swing.JSeparator;
import net.jcip.annotations.ThreadSafe;

/**
 * toolbar of the main view.
 */
@ThreadSafe
public class ToolbarView extends FrameView {

    private final ToolbarController controller;

    private JXButton openButton;
    private JXButton exportButton;
    private JXButton positiveXViewButton;
    private JXButton negativeXViewButton;
    private JXButton positiveYViewButton;
    private JXButton negativeYViewButton;
    
    /**
     * Creates new form ToolbarFrame
     * @throws java.lang.Exception
     */
    public ToolbarView() throws Exception {
        super();
        this.controller = new ToolbarController();
    }

    public final ToolbarController getController() {
        return controller;
    }

    @Override
    protected void initContainer() throws Exception {
        setLocationRelativeTo(getParent());
        setLayout(new FlowLayout(FlowLayout.LEFT));
    }

    @Override
    protected void initComponents() throws Exception {
        /* Init open button */
        openButton = new JXButton();
        openButton.setPreferredSize(new Dimension(35, 35));
        openButton.setIcon(loadIcon("/icons/open.png"));
        addLocaleCallback((locale) -> {
//            openButton.setText(bundle.getString("toolbarView_openButton"));
            openButton.setToolTipText(bundle.getString("toolbarView_openButton_tooltip"));
        });

        openButton.addActionListener((e) -> {
            final JFileChooser chooser = new JFileChooser();
            chooser.setFileFilter(new FileNameExtensionFilter(
                    bundle.getString("voxelSpaceFile_Name"), "vox"));
            chooser.setAcceptAllFileFilterUsed(false);

            int returnValue = chooser.showOpenDialog(parentView);

            if (returnValue == JFileChooser.APPROVE_OPTION) {
                final File selectedFile = chooser.getSelectedFile();

                if (controller != null) {
                    try {
                        final JProgressFrame progressFrame = new JProgressFrame();
                        progressFrame.setProgression(0, 1, "Import the voxel space file...");
                        
                        new Thread(() -> {
                            try {
                                controller.openVoxelSpaceFile(selectedFile, progressFrame);
                                progressFrame.dispose();
                            } catch (FileNotFoundException ex) {
                                Debugger.debugInterface(ex, DebugLevel.ERROR, parentView);
                            }
                        }).start();
                    } catch (Exception ex) {
                        Debugger.debugInterface(ex, DebugLevel.ERROR, parentView);
                    }
                }
            }
        });

        add(openButton);
        
        final JSeparator openSeparator = new JSeparator(JSeparator.VERTICAL);
        openSeparator.setPreferredSize(new Dimension(1, 35));
        add(openSeparator);
        
        /* Init export button */
        exportButton = new JXButton();
        exportButton.setPreferredSize(new Dimension(35, 35));
        exportButton.setIcon(loadIcon("/icons/export.png"));
        addLocaleCallback((locale) -> {
            exportButton.setToolTipText(bundle.getString("toolbarView_exportButton_tooltip"));
        });
        exportButton.addActionListener((e) -> {
            final JFileChooser chooser = new JFileChooser();
            chooser.setFileFilter(new FileNameExtensionFilter("Image", "png"));
            chooser.setAcceptAllFileFilterUsed(false);

            int returnValue = chooser.showSaveDialog(parentView);

            if (returnValue == JFileChooser.APPROVE_OPTION) {
                controller.exportScreenshot(chooser.getSelectedFile());
            }
        });
        add(exportButton);
        
        final JSeparator exportSeparator = new JSeparator(JSeparator.VERTICAL);
        exportSeparator.setPreferredSize(new Dimension(1, 35));
        add(exportSeparator);

        /* Init view buttons */
        positiveYViewButton = new JXButton();
        positiveYViewButton.setPreferredSize(new Dimension(35, 35));
        positiveYViewButton.setText(null);
        positiveYViewButton.setIcon(loadIcon("/icons/viewYpos.png"));
        positiveYViewButton.addActionListener((e) -> {
            controller.setCameraView(ViewType.Y_POS);
        });
        addLocaleCallback((locale) -> {
            positiveYViewButton.setToolTipText(bundle.getString("toolbarView_positiveYViewButton_tooltip"));
        });
        add(positiveYViewButton);
        
        negativeXViewButton = new JXButton();
        negativeXViewButton.setPreferredSize(new Dimension(35, 35));
        negativeXViewButton.setText(null);
        negativeXViewButton.setIcon(loadIcon("/icons/viewXneg.png"));
        negativeXViewButton.addActionListener((e) -> {
            controller.setCameraView(ViewType.X_NEG);
        });
        addLocaleCallback((locale) -> {
            negativeXViewButton.setToolTipText(bundle.getString("toolbarView_negativeXViewButton_tooltip"));
        });
        add(negativeXViewButton);
        
        negativeYViewButton = new JXButton();
        negativeYViewButton.setPreferredSize(new Dimension(35, 35));
        negativeYViewButton.setText(null);
        negativeYViewButton.setIcon(loadIcon("/icons/viewYneg.png"));
        negativeYViewButton.addActionListener((e) -> {
            controller.setCameraView(ViewType.Y_NEG);
        });
        addLocaleCallback((locale) -> {
            negativeYViewButton.setToolTipText(bundle.getString("toolbarView_negativeYViewButton_tooltip"));
        });
        add(negativeYViewButton);
        
        positiveXViewButton = new JXButton();
        positiveXViewButton.setPreferredSize(new Dimension(35, 35));
        positiveXViewButton.setText(null);
        positiveXViewButton.setIcon(loadIcon("/icons/viewXpos.png"));
        positiveXViewButton.addActionListener((e) -> {
            controller.setCameraView(ViewType.X_POS);
        });
        addLocaleCallback((locale) -> {
            positiveXViewButton.setToolTipText(bundle.getString("toolbarView_positiveXViewButton_tooltip"));
        });
        add(positiveXViewButton);
    }

}
